@@ -1,13 +1,12 @@
 import asyncio
 import logging
-from typing import Dict, Callable, List, Optional, Union, Pattern
+from typing import Dict, Callable, List, Optional, Union
 
-from .command import Command
-from .lexer import Lexer
-from .parser import Parser
 from .. import AsyncRunnable, MessageTypes, EventTypes  # interfaces & basics
 from .. import Cert, HTTPRequester, WebhookReceiver, WebsocketReceiver, Gateway, Client  # net related
 from .. import User, Channel, PublicChannel, PublicTextChannel, Guild, Event, Message  # concepts
+from ..command import CommandManager
+from ..task import TaskManager
 
 log = logging.getLogger(__name__)
 
@@ -16,12 +15,19 @@ class Bot(AsyncRunnable):
     """
     Represents a entity that handles msg/events and interact with users/khl server in manners that programmed.
     """
+    # components
     client: Client
+    command: CommandManager
+    task: TaskManager
+
+    # flags
+    _is_running: bool
+
+    # internal containers
     _me: Optional[User]
-    _cmd_index: Dict[str, Command]
     _event_index: Dict[EventTypes, List[Callable]]
 
-    def __init__(self, *, token: str = '', cert: Cert = None, client: Client = None, gate: Gateway = None,
+    def __init__(self, token: str = '', *, cert: Cert = None, client: Client = None, gate: Gateway = None,
                  out: HTTPRequester = None, compress: bool = True, port=5000, route='/khl-wh'):
         """
         The most common usage: ``Bot(token='xxxxxx')``
@@ -39,12 +45,18 @@ class Bot(AsyncRunnable):
         if not token and not cert:
             raise ValueError('require token or cert')
         self._init_client(cert or Cert(token=token), client, gate, out, compress, port, route)
-        self._me = None
-        self._cmd_index = {}
-        self._event_index = {}
         self.client.register(MessageTypes.TEXT, self._make_msg_handler())
         self.client.register(MessageTypes.SYS, self._make_event_handler())
 
+        self.command = CommandManager()
+        self.task = TaskManager()
+
+        self._is_running = False
+
+        self._me = None
+        self._event_index = {}
+        self._tasks = []
+
     def _init_client(self, cert: Cert, client: Client, gate: Gateway, out: HTTPRequester, compress: bool, port, route):
         """
         construct self.client from args.
@@ -84,14 +96,8 @@ class Bot(AsyncRunnable):
         construct a function to receive msg from client, and interpret it with _cmd_index
         """
 
-        # use closure
         async def handler(msg: Message):
-            for name, cmd in self._cmd_index.items():
-                try:
-                    args = cmd.prepare(msg)
-                except Lexer.LexerException:  # TODO: a more reasonable exception handle in lex and parse
-                    continue
-                await cmd.execute(msg, *args)
+            await self.command.handle(self.loop, msg, {Message: msg, Bot: self})
 
         return handler
 
@@ -106,52 +112,6 @@ class Bot(AsyncRunnable):
 
         return handler
 
-    def add_command(self, cmd: Command) -> Command:
-        """
-        register the cmd on current Bot
-
-        :param cmd: the Command going to be registered
-        :return: the cmd
-        """
-        if cmd.name in self._cmd_index:
-            raise ValueError(f'cmd: {cmd.name} already exists')
-        self._cmd_index[cmd.name] = cmd
-        log.debug(f'cmd: {cmd.name} added')
-        return cmd
-
-    def get_command(self, name: str) -> Optional[Command]:
-        return self._cmd_index.get(name, None)
-
-    def remove_command(self, name: str):
-        if name in self._cmd_index:
-            del self._cmd_index[name]
-
-    def command(self, name: str = '', *, help: str = '', desc: str = '',
-                aliases: List[str] = (), prefixes: List[str] = ('/',), regex: Union[str, Pattern] = '',
-                lexer: Lexer = None, parser: Parser = None, rules: List[Callable] = ()):
-        """
-        decorator, wrap a function in Command and register it on current Bot
-
-        :param name: the name of this Command, also used to trigger command in DefaultLexer
-        :param aliases: (DefaultLexer only) you can also trigger the command with aliases
-        :param prefixes: (DefaultLexer only) command prefix, default use '/'
-        :param regex: (RELexer only) pattern for the command
-        :param help: detailed manual
-        :param desc: short introduction
-        :param lexer: (Advanced) explicitly set the lexer
-        :param parser: (Advanced) explicitly set the parser
-        :param rules: only be executed if all rules are met
-        :return: wrapped Command
-        """
-        args = {'help': help, 'desc': desc,
-                'aliases': aliases, 'prefixes': prefixes, 'regex': regex,
-                'lexer': lexer, 'parser': parser, 'rules': rules}
-
-        # use lambda cuz i do not wanna declare decorator() explicitly to take 3 blank lines
-        # did not init Lexer in advance cuz it needs func.__name__
-        # this is redundant stuff in constructor, there should be a better way
-        return lambda func: self.add_command(Command.command(name, **args)(func))
-
     def add_event_handler(self, type: EventTypes, handler: Callable):
         if type not in self._event_index:
             self._event_index[type] = []
@@ -166,13 +126,12 @@ class Bot(AsyncRunnable):
         :param type: the type
         :return: original func
         """
-
         return lambda func: self.add_event_handler(type, func)
 
     async def fetch_me(self, force_update: bool = False) -> User:
         """fetch detail of the bot it self as a ``User``"""
-        if force_update or not self._me or not self._me.is_loaded():
-            self._me = await self.client.fetch_me()
+        if not self._me or not self._me.is_loaded():
+            self._me = await self.client.fetch_me(force_update)
         return self._me
 
     @property
@@ -238,19 +197,55 @@ class Bot(AsyncRunnable):
             raise ValueError('can not modify guild from other gate')
         return await guild.leave()
 
-    async def delete_message(self, msg: Message):
-        """delete msg"""
-        if msg.gate.requester != self.client.gate.requester:
-            raise ValueError('can not modify message from other gate')
-        return msg.delete()
+    async def _make_temp_msg(self, msg_id: str) -> Message:
+        return Message(msg_id=msg_id, _gate_=self.client.gate)
+
+    async def delete_message(self, msg: Union[Message, str]):
+        """delete msg
+
+        wraps `Message.delete`
+
+        :param msg: the msg, accepts Message and msg_id(str)
+        """
+        if isinstance(msg, str):
+            msg = self._make_temp_msg(msg)
+        return await msg.delete()
+
+    async def add_reaction(self, msg: Union[Message, str], emoji: str):
+        """add emoji to msg's reaction list
+
+        wraps `Message.add_reaction`
+
+        :param msg: accepts `Message` and msg_id(str)
+        :param emoji: 😘
+        """
+        if isinstance(msg, str):
+            msg = self._make_temp_msg(msg)
+        return await msg.add_reaction(emoji)
+
+    async def delete_reaction(self, msg: Union[Message, str], emoji: str, user: User = None):
+        """delete emoji from msg's reaction list
+
+        wraps `Message.delete_reaction`
+
+        :param msg: accepts `Message` and msg_id(str)
+        :param emoji: 😘
+        :param user: whose reaction, delete others added reaction requires channel msg admin permission
+        """
+        if isinstance(msg, str):
+            msg = self._make_temp_msg(msg)
+        return await msg.delete_reaction(emoji, user)
+
+    async def start(self):
+        if self._is_running:
+            raise RuntimeError('this bot is already running')
+        self.task.schedule()
+        await self.client.start()
 
     def run(self):
+        if not self.loop:
+            self.loop = asyncio.get_event_loop()
         try:
-            if not self.loop:
-                self.loop = asyncio.get_event_loop()
-            self.loop.run_until_complete(self.client.run())
+            self.loop.run_until_complete(self.start())
         except KeyboardInterrupt:
-            pass
-        except Exception as e:
-            log.error(e)
-        log.info('see you next time')
+            log.info('see you next time')
