@@ -1,7 +1,7 @@
 import asyncio
 import inspect
 import logging
-from typing import Dict, List, Callable
+from typing import Dict, List, Callable, Coroutine
 
 from . import api
 from .channel import public_channel_factory, PublicChannel
@@ -13,6 +13,8 @@ from .user import User
 
 log = logging.getLogger(__name__)
 
+TypeHandler = Callable[..., Coroutine]
+
 
 class Client(Requestable, AsyncRunnable):
     """
@@ -22,20 +24,22 @@ class Client(Requestable, AsyncRunnable):
 
     reminder: Client.loop only used to run handle_event() and registered handlers.
     """
-    _handler_map: Dict[MessageTypes, List[Callable]]
+    _handler_map: Dict[MessageTypes, List[TypeHandler]]
 
     def __init__(self, gate: Gateway):
         self.gate = gate
+        self.ignore_self_msg = True
+        self._me = None
 
         self._handler_map = {}
         self._pkg_queue = asyncio.Queue()
 
-    def register(self, type: MessageTypes, handler: Callable):
+    def register(self, type: MessageTypes, handler: TypeHandler):
         if not asyncio.iscoroutinefunction(handler):
             raise TypeError('handler must be a coroutine.')
 
-        params = list(inspect.signature(handler).parameters.items())
-        if len(params) != 1 or not issubclass(params[0][1].annotation, RawMessage):
+        params = list(inspect.signature(handler).parameters.values())
+        if len(params) != 1 or not issubclass(params[0].annotation, RawMessage):
             raise TypeError('handler must have one and only one param, and the param inherits RawMessage')
 
         if type not in self._handler_map:
@@ -44,22 +48,31 @@ class Client(Requestable, AsyncRunnable):
 
     async def handle_pkg(self):
         """
-        Pop `pkg` from `event_queue`,
-        spawn `msg` according to `pkg`,
-        pass `msg` to corresponding handlers defined in `_handler_map`
+        consume `pkg` from `event_queue`
         """
         while True:
             pkg: Dict = await self._pkg_queue.get()
             log.debug(f'upcoming pkg: {pkg}')
 
             try:
-                msg = self._make_msg(pkg)
-                self._dispatch_msg(msg)
+                await self._consume_pkg(pkg)
             except Exception as e:
                 log.exception(e)
 
             self._pkg_queue.task_done()
 
+    async def _consume_pkg(self, pkg: Dict):
+        """
+        spawn `msg` according to `pkg`,
+        check if ignore msgs from self
+        pass `msg` to corresponding handlers defined in `_handler_map`
+        """
+        msg = self._make_msg(pkg)
+        if self.ignore_self_msg and msg.type != MessageTypes.SYS:
+            if msg.author.id == (await self.fetch_me()).id:
+                return
+        self._dispatch_msg(msg)
+
     def _make_msg(self, pkg: Dict):
         if pkg.get('type') == MessageTypes.SYS.value:
             msg = Event(**pkg)
@@ -83,15 +96,27 @@ class Client(Requestable, AsyncRunnable):
             return
         if msg.type in self._handler_map and self._handler_map[msg.type]:
             for handler in self._handler_map[msg.type]:
-                asyncio.ensure_future(handler(msg), loop=self.loop)
+                asyncio.ensure_future(self._handle_safe(handler)(msg), loop=self.loop)
+
+    @staticmethod
+    def _handle_safe(handler: TypeHandler):
+        async def safe_handler(msg):
+            try:
+                await handler(msg)
+            except Exception as e:
+                log.exception(f'error raised during message handling', exc_info=e)
+
+        return safe_handler
 
     async def create_asset(self, file: str) -> str:
         """upload ``file`` to khl, and return the url to the file"""
         return (await self.gate.exec_req(api.Asset.create(file=open(file, 'rb'))))['url']
 
-    async def fetch_me(self) -> User:
+    async def fetch_me(self, force_update: bool = False) -> User:
         """fetch detail of the ``User`` on the client"""
-        return User(_gate_=self.gate, _lazy_loaded_=True, **(await self.gate.exec_req(api.User.me())))
+        if force_update or not self._me or not self._me.is_loaded():
+            self._me = User(_gate_=self.gate, _lazy_loaded_=True, **(await self.gate.exec_req(api.User.me())))
+        return self._me
 
     async def fetch_public_channel(self, channel_id: str) -> PublicChannel:
         """fetch details of a public channel from khl"""
@@ -103,6 +128,5 @@ class Client(Requestable, AsyncRunnable):
         guilds_data = (await self.gate.exec_pagination_req(api.Guild.list()))
         return [Guild(_gate_=self.gate, _lazy_loaded_=True, **i) for i in guilds_data]
 
-    async def run(self):
-        asyncio.ensure_future(self.handle_pkg(), loop=self.loop)
-        await self.gate.run(self._pkg_queue)
+    async def start(self):
+        await asyncio.gather(self.handle_pkg(), self.gate.run(self._pkg_queue))
