@@ -12,12 +12,22 @@ class AsyncRunnable(ABC):
 
     @property
     def loop(self) -> asyncio.AbstractEventLoop:
+        """the event loop for the async work"""
         return self._loop
 
     @loop.setter
     def loop(self, new_loop: asyncio.AbstractEventLoop):
         self._loop = new_loop
 
+    def schedule(self):
+        """schedule the async work into background"""
+        asyncio.ensure_future(self.start(), loop=self.loop)
+
+    @abstractmethod
+    async def start(self):
+        """run the async work"""
+        ...
+
 
 class LazyLoadable(ABC):
     """
@@ -117,6 +127,6 @@ class EventTypes(Enum):
 
     JOINED_CHANNEL = 'joined_channel'
     EXITED_CHANNEL = 'exited_channel'
-    USER_UPDATE = 'user_update'
+    USER_UPDATED = 'user_updated'
     SELF_JOINED_GUILD = 'self_joined_guild'
     SELF_EXITED_GUILD = 'self_exited_guild'
