@@ -1,5 +1,4 @@
 import asyncio
-import json
 import logging
 import time
 import zlib
@@ -17,6 +16,8 @@ API = f'https://www.kaiheila.cn/api/v3'
 
 
 class Receiver(AsyncRunnable, ABC):
+    _queue: asyncio.Queue
+
     @abstractmethod
     def __init__(self):
         raise NotImplementedError
@@ -25,8 +26,16 @@ class Receiver(AsyncRunnable, ABC):
     def type(self) -> str:
         raise NotImplementedError
 
+    @property
+    def pkg_queue(self) -> asyncio.Queue:
+        return self._queue
+
+    @pkg_queue.setter
+    def pkg_queue(self, queue: asyncio.Queue):
+        self._queue = queue
+
     @abstractmethod
-    async def run(self, pkg_queue: asyncio.Queue):
+    async def start(self):
         raise NotImplementedError
 
 
@@ -45,15 +54,13 @@ class WebsocketReceiver(Receiver):
 
     async def heartbeat(self, ws_conn: ClientWebSocketResponse):
         while True:
-            await asyncio.sleep(26)
-            await ws_conn.send_json({'s': 2, 'sn': self._NEWEST_SN})
+            try:
+                await asyncio.sleep(26)
+                await ws_conn.send_json({'s': 2, 'sn': self._NEWEST_SN})
+            except Exception as e:
+                log.exception('error raised during websocket heartbeat', exc_info=e)
 
-    def __raw_to_pkg(self, data: bytes) -> dict:
-        data = zlib.decompress(data) if self.compress else data
-        data = json.loads(str(data, encoding='utf-8'))
-        return data
-
-    async def run(self, pkg_queue: asyncio.Queue):
+    async def start(self):
         async with ClientSession(loop=self.loop) as cs:
             headers = {
                 'Authorization': f'Bot {self._cert.token}',
@@ -76,26 +83,26 @@ class WebsocketReceiver(Receiver):
                 log.info('[ init ] launched')
 
                 async for raw in ws_conn:
+                    raw: WSMessage
                     try:
-                        raw: WSMessage
-                        pkg: Dict = self.__raw_to_pkg(raw.data)
+                        data = raw.data
+                        data = zlib.decompress(data) if self.compress else data
+                        pkg: Dict = self._cert.decode_raw(data)
                         log.debug(f'upcoming raw: {pkg}')
+                        if pkg['s'] != 0:
+                            continue
+                        self._NEWEST_SN = pkg['sn']
+                        await self.pkg_queue.put(pkg['d'])
                     except Exception as e:
                         log.exception(e)
                         continue
 
-                    if pkg['s'] == 0:
-                        self._NEWEST_SN = pkg['sn']
-                        event = pkg['d']
-                        await pkg_queue.put(event)
-
 
 class WebhookReceiver(Receiver):
     def __init__(self, cert: Cert, *, port: int, route: str, compress: bool):
         self._cert = cert
         self.port = port
         self.route = route
-        self.cs = ClientSession()
         self.app = web.Application()
         self.compress = compress
         self.sn_dup_map = {}
@@ -104,15 +111,10 @@ class WebhookReceiver(Receiver):
     def type(self) -> str:
         return 'webhook'
 
-    def __raw_to_pkg(self, data: bytes) -> dict:
-        data = zlib.decompress(data) if self.compress else data
-        data = json.loads(str(data, encoding='utf-8'))
-        return ('encrypt' in data.keys()) and json.loads(self._cert.decrypt(data['encrypt'])) or data
-
-    def __is_pkg_dup(self, req: dict) -> bool:
+    def _is_dup(self, req: dict) -> bool:
         # TODO: need a recycle count down ring
         sn = req.get('sn', None)
-        if not sn:
+        if sn is None:
             return False
         current = time.time()
         if sn in self.sn_dup_map.keys():
@@ -122,38 +124,36 @@ class WebhookReceiver(Receiver):
         self.sn_dup_map[sn] = current
         return False
 
-    async def run(self, pkg_queue: asyncio.Queue):
+    async def start(self):
         async def on_recv(request: web.Request):
             try:
-                pkg: Dict = self.__raw_to_pkg(await request.read())
+                data = await request.read()
+                data = zlib.decompress(data) if self.compress else data
+                pkg: Dict = self._cert.decode_raw(data)
             except Exception as e:
                 log.exception(e)
                 return web.Response()
 
-            if not pkg or pkg['d']['verify_token'] != self._cert.verify_token:
-                # empty pkg or verify_token check failed
+            if not pkg:  # empty pkg
                 return web.Response()
 
-            if self.__is_pkg_dup(pkg):
-                # dupe pkg
+            if pkg['d']['verify_token'] != self._cert.verify_token:  # check verify_token
+                return web.Response()
+
+            if self._is_dup(pkg):  # dup pkg
                 return web.Response()
 
             if pkg['s'] == 0:
                 pkg = pkg['d']
                 if pkg['type'] == 255 and pkg['channel_type'] == 'WEBHOOK_CHALLENGE':
                     return web.json_response({'challenge': pkg['challenge']})
-                await pkg_queue.put(pkg)
+                await self.pkg_queue.put(pkg)
 
             return web.Response()
 
         self.app.router.add_post(self.route, on_recv)
 
-        async def on_shutdown(app):
-            await self.cs.close()
-
-        self.app.on_shutdown.append(on_shutdown)
-
-        runner = web.AppRunner(self.app, access_log_class=None)
+        runner = web.AppRunner(self.app)
         await runner.setup()  # runner use its own loop, can not be set
         site = web.TCPSite(runner, '0.0.0.0', self.port)
 
